# loader.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# Based on djangojinja2.py by Armin Ronacher.

'''
This module provides a partially-compatible replacement for
`django.template.loader`. Included functions are `get_template`,
`select_template`, and `render_to_string`. The former two are useful for
loading template objects, whereas the latter is useful for rendering a
template given a context.

'''

from django.template.context import get_standard_processors
import jinja2

from chouwa.environment import env

class TemplateSyntaxError(jinja2.TemplateSyntaxError):
    def __init__(self, message, lineno, name, filename):
        jinja2.TemplateError.__init__(self, message)
        self.lineno = lineno
        self.name = name
        self.filename = filename

    def __unicode__(self):
        location = 'line %d' % self.lineno
        name = self.filename or self.name
        if name:
            location = 'File "%s", %s' % (name, location)
        return u'\n'.join([self.message, '  ' + location])

def get_template(template_name, globals=None):
    """Load a template."""
    return env.get_template(template_name, globals=globals)

def select_template(templates, globals=None):
    """Try to load one of the given templates."""
    for template in templates:
        return env.get_template(template, globals=globals)
    raise jinja2.TemplateNotFound(', '.join(templates))

def render_to_string(template_name, context=None, request=None):
    """Render a template into a string."""
    if context is None:
        context = {}
    if request is not None:
        context['request'] = request
        for processor in get_standard_processors():
            context.update(processor(request))
    try:
        return get_template(template_name).render(context)
    except jinja2.TemplateSyntaxError, e:
        raise TemplateSyntaxError(e.message, e.lineno, e.name, e.filename)



# defaultglobals.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
This module provides a few filters and globals similar to those shipped
with django but not present in jinja2 by default.

'''

from datetime import datetime

from django.core.urlresolvers import reverse
from django.utils.dateformat import format

from chouwa.decorators import jinjaglobal, jinjafilter

@jinjaglobal
def url(viewname, *args, **kwargs):
    '''
    Exactly proxy the API of `django.core.urlresolvers.reverse`.

    :Parameters:
      viewname : str
        The label associated with a url or the dotted path module name
        of a view.
    '''

    return reverse(viewname, *args, **kwargs)

@jinjaglobal
def url2(viewname, *args, **kwargs):
    '''
    Lookup a url via `django.core.urlresolvers.reverse`. Positional and
    keyword arguments to this function are used directly as captures in
    the URL.

    Use `url` when you need to specify a prefix or urlconf for
    ``reverse`` or if you have a URL capture named ``viewname``.

    :Parameters:
      viewname : str
        The label associated with a url or the dotted path module name
        of a view.
    '''

    return reverse(viewname, args=args, kwargs=kwargs)

@jinjaglobal
def now(format_string):
    '''
    Displays the date, formatted according to the given string.

    Format specifiers are identical to those understood by PHP's
    ``date()`` function. See <http://php.net/date>.

    Sample usage::

        It is {{ now("F jS, Y H:i") }}.

    Based on `django.template.defaulttags.now`.

    :Parameters:
      format_string : str
        The PHP ``date()``-style format specifier.

    '''

    return format(datetime.now(), format_string)

@jinjafilter
def date(dt, format_string):
    '''
    Format a date using the PHP ``date()`` format specifiers. Similar to
    the `now` function, except that it formats an arbitrary date.

    :Parameters:
      dt : `datetime.datetime`
        The datetime object to format.
      format_string : str
        The PHP ``date()``-style format specifier.

    '''

    return format(dt, format_string)



# environment.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
Create a Jinja2 Environment object and install filters and globals.

Templates will be searched for by default in template subdirectories
of installed applications and then in directories listed in the
``TEMPLATE_DIRS`` setting.

The default environment settings can be overridden by creating a Django
setting ``JINJA2_ENVIRONMENT_OPTIONS``, which is a dictionary of
parameter to pass to `jinja2.Environment`. For example, to enable strict
undefined handling, one could set:

.. python::
    from jinja2 import StrictUndefined

    JINJA2_ENVIRONMENT_OPTIONS = {'undefined': StrictUndefined}

'''

from itertools import chain
import os
import sys

from django.conf import settings
from django.template.loaders.app_directories import app_template_dirs
from django.utils import translation
from jinja2 import Environment, FileSystemLoader
from jinja2.ext import i18n

from chouwa import defaultglobals

class DjangoTranslator(object):
    def ugettext(self, *args):
        return translation.ugettext(*args)

    def ungettext(self, *args):
        return translation.ungettext(*args)

def get_app_modules():
    '''
    Generator yielding module objects containing application-specific
    jinjaglobals. Note that the globals are not namespaced for the
    application.

    '''

    for app_label in settings.INSTALLED_APPS:
        mod_name = '.'.join((app_label, 'jinjaglobals'))
        try:
            __import__(mod_name, {}, {}, [], 0)
            yield sys.modules[mod_name]
        except ImportError:
            pass

def install_globals(env):
    '''
    Add the default filters and globals to the jinja2 environment.

    :Parameters:
      env : `jinja2.Environment`
        The jinja2 environment.

    '''

    for mod in chain((defaultglobals,), get_app_modules()):
        for name in dir(mod):
            global_ = getattr(mod, name)
            if getattr(global_, 'is_jinja_global', False):
                env.globals[name] = global_
            elif getattr(global_, 'is_jinja_filter', False):
                env.filters[name] = global_
            elif getattr(global_, 'is_jinja_test', False):
                env.tests[name] = global_

def make_environment():
    '''
    Create and populate the jinja2 environment.

    '''

    searchpath = app_template_dirs + settings.TEMPLATE_DIRS
    env_options = {'loader': FileSystemLoader(searchpath),
                   'autoescape': True, 'extensions': (i18n,),
                   'auto_reload': settings.TEMPLATE_DEBUG}
    env_options.update(getattr(settings, 'JINJA2_ENVIRONMENT_OPTIONS', {}))
    env = Environment(**env_options)
    install_globals(env)
    env.install_gettext_translations(DjangoTranslator())
    return env

#: The global jinja2 Environment object.
env = make_environment()



# __init__.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
Chouwa is a library which aims to ease the integration of jinja2 into
django projects. It does this by providing equivalents of some of
django's helper methods which depend on its template subsystem, as well
as providing builtin globals and filters and providing a mechanism for
user applications to introduce their own globals and filters.

To install chouwa, clone the repository to your python path. The name of
the directory containing the python modules should be ``chouwa``.

Basic usage of chouwa is simple. Consider the use case of rendering a
template within a django application from a view and returning the
result as a `django.http.HTTPResponse`. This use case corresponds to the
`django.views.generic.simple.direct_to_template` helper function. Using
chouwa, the corresponding idiom is as follows:

.. python::

    from chouwa.shortcuts import direct_to_template
    from django.shortcuts import get_object_or_404

    from myapp.models import Entry

    def entry_detail(request, entry_id):
        entry = get_object_or_404(Entry, id=entry_id)
        return direct_to_template(request, 'myapp/entry_detail.html',
                                  {'entry': entry})

`jinja`_ supports similar functionality to django filters and tags by
inserting python functions into the context and either calling them or
applying the filter syntax to them. `Custom filters and global
functions`_ are supported by creating a python modules named
``jinjaglobals`` inside your django application packages. For further
documentation on the creation of custom filters and globals, see the
`chouwa.decorators` documentation.

.. _`jinja`: http://jinja.pocoo.org/2/documentation/
.. _`Custom filters and global functions`:
   http://jinja.pocoo.org/2/documentation/api#custom-filters

'''

__version__ = (0, 2, 0)



# views.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
The following views are provided to facilitate the use of jinja
templates for django error pages. They serve as replacements for
`django.views.defaults.page_not_found` and
`django.views.defaults.server_error`.

To use these views in your project, add the following line to your root
URLconf after ``from django.conf.urls.defaults import *``. The module
`django.conf.urls.defaults` includes default 404 and 500 handlers, thus
it is necessary to import the chouwa handlers after an ``import *`` from
the django module.

.. python::

    from chouwa.views import handler404, handler500

'''

from django.http import HttpResponseNotFound, HttpResponseServerError

from chouwa.loader import get_template

handler404 = 'chouwa.views.page_not_found'
handler500 = 'chouwa.views.server_error'

def page_not_found(request, template_name='404.html'):
    t = get_template(template_name)
    return HttpResponseNotFound(t.render({'request_path': request.path}))

def server_error(request, template_name='500.html'):
    t = get_template(template_name)
    return HttpResponseServerError(t.render())



# shortcuts.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
It is often more convienient to defer the actual rendering of templates
to helper functions rather than manipulate template objects oneself.
This module offers shortcut functions for common use cases involving
template rendering. For finer-grained control, see `chouwa.loader`.

'''

from django.http import HttpResponse

from chouwa.loader import render_to_string

def direct_to_template(request, template, extra_context=None,
                       mimetype=None):
    '''
    Render a template and return the result in a
    `django.http.HTTPResponse`. The context of the template rendering
    is populated by applying the standard context processors, and then
    updating the context with the values supplied in `extra_context`.
    Similar to `django.views.generic.simple.direct_to_template`.

    :Parameters:
      request : `django.http.HTTPRequest`
        The request object from django.
      template : str
        The path of the template relative to one of the template
        directories.
      extra_context : dict
        A dictionary containing all extra items to add to the template
        context.
      mimetype : str
        The Content-type to send to the client.

    '''

    return HttpResponse(render_to_string(template, extra_context,
                                         request=request),
                        mimetype=mimetype)



# decorators.py
#
# Copyright (c) 2009 Trevor Caira
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

'''
Applications can add their own global objects and filters to the jinja2
environment. This is accomplished by creating a module ``jinjaglobals`` 
inside an installed application, and marking the desired globals in that
module with one of the decorators below. Note that the globals and
filters are not actually namespaced by application: all applications can
see globals added by any application. For an example, see below:

.. python::

    from time import time as time_

    from chouwa.decorators import jinjaglobal, jinjafilter

    @jinjafilter
    def add_five(value):
        return value + 5

    @jinjaglobal
    def time():
        return time_()

This example adds a filter `add_five` which retuns the value passed to
it with 5 added, and adds a global `time` which returns the current unix
timestamp.

'''

def jinjaglobal(function):
    '''
    Mark a callable as a jinja global.

    '''

    function.is_jinja_global = True
    return function

def jinjafilter(function):
    '''
    Mark a callable as a `jinja filter`_.

    _`jinja filter`: http://jinja.pocoo.org/2/documentation/api#custom-filters

    '''

    function.is_jinja_filter = True
    return function

def jinjatest(function):
    '''
    Mark a callable as a `jinja test`_.

    _`jinja test`: http://jinja.pocoo.org/2/documentation/api#writing-tests

    '''

    function.is_jinja_test = True
    return function



from setuptools import setup

from chouwa import __version__

setup(
    name='chouwa',
    version='.'.join(str(bit) for bit in __version__),
    url='http://bitbucket.org/trevor/chouwa',
    description='Django & Jinja2 integration',
    author='Trevor Caira',
    author_email='trevor@caira.com',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    packages=['chouwa'],
)
